@@ -135,7 +135,7 @@ typedef struct MySQLFdwExecState
 	char	   *query;			/* Query string */
 	Relation	rel;			/* relcache entry for the foreign table */
 	List	   *retrieved_attrs;	/* list of target attribute numbers */
-	bool		cursor_exists;	/* have we created the cursor? */
+	bool		query_executed;	/* have we executed the query? */
 	int			numParams;		/* number of parameters passed to query */
 	FmgrInfo   *param_flinfo;	/* output conversion functions for them */
 	List	   *param_exprs;	/* executable expressions for param values */
@@ -161,8 +161,6 @@ typedef struct MySQLColumn
 	int			atttype;		/* Attribute type */
 } MySQLColumn;
 
-extern bool is_foreign_expr(PlannerInfo *root, RelOptInfo *baserel,
-							Expr *expr);
 
 extern int ((mysql_options) (MYSQL *mysql, enum mysql_option option,
 							 const void *arg));
@@ -229,6 +227,8 @@ extern void mysql_append_where_clause(StringInfo buf, PlannerInfo *root,
 									  RelOptInfo *baserel, List *exprs,
 									  bool is_first, List **params);
 extern void mysql_deparse_analyze(StringInfo buf, char *dbname, char *relname);
+extern bool mysql_is_foreign_expr(PlannerInfo *root, RelOptInfo *baserel,
+								  Expr *expr);
 
 
 /* connection.c headers */
