@@ -222,6 +222,74 @@ SELECT d.c1, d.c2, e.c1, e.c2, e.c6, e.c8
 SELECT d.c1, d.c2, e.c1, e.c2, e.c6, e.c8
   FROM f_test_tbl2 d FULL OUTER JOIN l_test_tbl1 e ON d.c1 = e.c8 ORDER BY 1, 3;
 
+-- FDW-206: LEFT JOIN LATERAL case should not crash
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT * FROM f_mysql_test t1 LEFT JOIN LATERAL (
+  SELECT t2.a, t1.a AS t1_a FROM f_mysql_test t2) t3 ON t1.a = t3.a ORDER BY 1;
+SELECT * FROM f_mysql_test t1 LEFT JOIN LATERAL (
+  SELECT t2.a, t1.a AS t1_a FROM f_mysql_test t2) t3 ON t1.a = t3.a ORDER BY 1;
+SELECT t1.c1, t3.c1, t3.t1_c8 FROM f_test_tbl1 t1 INNER JOIN LATERAL (
+  SELECT t2.c1, t1.c8 AS t1_c8 FROM f_test_tbl2 t2) t3 ON t3.c1 = t3.t1_c8
+  ORDER BY 1, 2, 3;
+SELECT t1.c1, t3.c1, t3.t1_c8 FROM l_test_tbl1 t1 LEFT JOIN LATERAL (
+  SELECT t2.c1, t1.c8 AS t1_c8 FROM f_test_tbl2 t2) t3 ON t3.c1 = t3.t1_c8
+  ORDER BY 1, 2, 3;
+SELECT *, (SELECT r FROM (SELECT c1 AS c1) x, LATERAL (SELECT c1 AS r) y)
+  FROM f_test_tbl1 ORDER BY 1, 2, 3;
+-- LATERAL JOIN with RIGHT should throw error
+SELECT t1.c1, t3.c1, t3.t1_c8 FROM f_test_tbl1 t1 RIGHT JOIN LATERAL (
+  SELECT t2.c1, t1.c8 AS t1_c8 FROM f_test_tbl2 t2) t3 ON t3.c1 = t3.t1_c8
+  ORDER BY 1, 2, 3;
+
+-- FDW-207: NATURAL JOIN should give correct output
+SELECT t1.c1, t2.c1, t3.c1
+  FROM f_test_tbl1 t1 NATURAL JOIN f_test_tbl1 t2 NATURAL JOIN f_test_tbl1 t3
+  ORDER BY 1, 2, 3;
+
+-- FDW-208: IS NULL and LIKE should give the correct output with
+-- use_remote_estimate set to true.
+INSERT INTO f_test_tbl2 VALUES (50, 'TEMP1', NULL);
+INSERT INTO f_test_tbl2 VALUES (60, 'TEMP2', NULL);
+ALTER SERVER mysql_svr OPTIONS (use_remote_estimate 'true');
+SELECT t1.c1, t2.c1
+  FROM f_test_tbl2 t1 INNER JOIN f_test_tbl2 t2 ON t1.c1 = t2.c1
+  WHERE t1.c3 IS NULL ORDER BY 1, 2;
+SELECT t1.c1, t2.c1
+  FROM f_test_tbl2 t1 INNER JOIN f_test_tbl2 t2 ON t1.c1 = t2.c1 AND t1.c2 LIKE 'TEMP%'
+  ORDER BY 1, 2;
+DELETE FROM f_test_tbl2 WHERE c1 IN (50, 60);
+ALTER SERVER mysql_svr OPTIONS (SET use_remote_estimate 'false');
+
+-- FDW-169: Insert/Update/Delete on enum column.
+INSERT INTO f_enum_t1
+  VALUES (1, 'small'), (2, 'medium'), (3, 'medium'), (4, 'small');
+SELECT * FROM f_enum_t1 WHERE id = 4;
+UPDATE f_enum_t1 SET size = 'large' WHERE id = 4;
+SELECT * FROM f_enum_t1 WHERE id = 4;
+DELETE FROM f_enum_t1 WHERE size = 'large';
+SELECT * FROM f_enum_t1 WHERE id = 4;
+
+-- Negative scenarios for ENUM handling.
+-- Test that if we insert the ENUM value which is not present on MySQL side,
+-- but present on Postgres side.
+DROP FOREIGN TABLE f_enum_t1;
+DROP TYPE size_t;
+-- Create the type with extra enum values.
+CREATE TYPE size_t AS enum('small', 'medium', 'large', 'largest', '');
+CREATE FOREIGN TABLE f_enum_t1(id int, size size_t)
+  SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'enum_t1');
+
+-- If we insert the enum value which is not present on MySQL side then it
+-- inserts empty string in ANSI_QUOTES sql_mode, so verify that.
+INSERT INTO f_enum_t1 VALUES (4, 'largest');
+SELECT * from f_enum_t1;
+DELETE FROM f_enum_t1 WHERE size = '';
+
+-- Postgres should throw an error as the value which we are inserting for enum
+-- column is not present in enum on Postgres side, no matter whether it is
+-- present on MySQL side or not. PG's sanity check itself throws an error.
+INSERT INTO f_enum_t1 VALUES (4, 'big');
+
 -- FDW-155: Enum data type can be handled correctly in select statements on
 -- foreign table.
 SELECT * FROM f_enum_t1 WHERE size = 'medium' ORDER BY id;
@@ -232,14 +300,66 @@ EXPLAIN (VERBOSE, COSTS OFF)
 SELECT EXISTS(SELECT 1 FROM pg_enum), sum(id) from f_enum_t1;
 SELECT EXISTS(SELECT 1 FROM pg_enum), sum(id) from f_enum_t1;
 
--- Check with IMPORT SCHEMA command.
-IMPORT FOREIGN SCHEMA mysql_fdw_regress LIMIT TO (enum_t1) FROM SERVER mysql_svr INTO public;
+-- Check with the IMPORT FOREIGN SCHEMA command.  Also, check ENUM types with
+-- the IMPORT FOREIGN SCHEMA command. If the enum name is the same for multiple
+-- tables, then it should handle correctly by prefixing the table name.
+CREATE TYPE enum_t1_size_t AS enum('small', 'medium', 'large');
+CREATE TYPE enum_t2_size_t AS enum('S', 'M', 'L');
+IMPORT FOREIGN SCHEMA mysql_fdw_regress LIMIT TO (enum_t1, enum_t2)
+  FROM SERVER mysql_svr INTO public;
+SELECT attrelid::regclass, atttypid::regtype FROM pg_attribute
+  WHERE (attrelid = 'enum_t1'::regclass OR attrelid = 'enum_t2'::regclass) AND
+    attnum > 1 ORDER BY 1;
 SELECT * FROM enum_t1 ORDER BY id;
+SELECT * FROM enum_t2 ORDER BY id;
 DROP FOREIGN TABLE enum_t1;
+DROP FOREIGN TABLE enum_t2;
+
+-- Parameterized queries should work correctly.
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT c1, c2 FROM f_test_tbl1
+  WHERE c8 = (SELECT c1 FROM f_test_tbl2 WHERE c1 = (SELECT 20))
+  ORDER BY c1;
+SELECT c1, c2 FROM f_test_tbl1
+  WHERE c8 = (SELECT c1 FROM f_test_tbl2 WHERE c1 = (SELECT 20))
+  ORDER BY c1;
+
+SELECT * FROM f_test_tbl1
+  WHERE c8 NOT IN (SELECT c1 FROM f_test_tbl2 WHERE c1 = (SELECT 20))
+  ORDER BY c1;
+
+-- Check parameterized queries with text/varchar column, should not crash.
+CREATE FOREIGN TABLE f_test_tbl3 (c1 INTEGER, c2 text, c3 text)
+  SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test_tbl2');
+CREATE TABLE local_t1 (c1 INTEGER, c2 text);
+INSERT INTO local_t1 VALUES (1, 'SALES');
+
+SELECT c1, c2 FROM f_test_tbl3 WHERE c3 = (SELECT 'PUNE'::text) ORDER BY c1;
+SELECT c1, c2 FROM f_test_tbl2 WHERE c3 = (SELECT 'PUNE'::varchar) ORDER BY c1;
+
+SELECT * FROM local_t1 lt1 WHERE lt1.c1 =
+  (SELECT count(*) FROM f_test_tbl3 ft1 WHERE ft1.c2 = lt1.c2) ORDER BY lt1.c1;
+
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT c1, c2 FROM f_test_tbl1 WHERE c8 = (
+  SELECT c1 FROM f_test_tbl2 WHERE c1 = (
+    SELECT min(c1) + 1 FROM f_test_tbl2)) ORDER BY c1;
+SELECT c1, c2 FROM f_test_tbl1 WHERE c8 = (
+  SELECT c1 FROM f_test_tbl2 WHERE c1 = (
+    SELECT min(c1) + 1 FROM f_test_tbl2)) ORDER BY c1;
+
+SELECT * FROM f_test_tbl1 WHERE c1 = (SELECT 500) AND c2 = (
+  SELECT max(c2) FROM f_test_tbl1 WHERE c4 = (SELECT 600))
+  ORDER BY 1, 2;
+SELECT t1.c1, (SELECT c2 FROM f_test_tbl1 WHERE c1 =(SELECT 500))
+  FROM f_test_tbl2 t1, (
+    SELECT c1, c2 FROM f_test_tbl2 WHERE c1 > ANY (SELECT 20)) t2
+  ORDER BY 1, 2;
 
 -- Cleanup
 DROP TABLE l_test_tbl1;
 DROP TABLE l_test_tbl2;
+DROP TABLE local_t1;
 DROP VIEW smpl_vw;
 DROP VIEW comp_vw;
 DROP VIEW ttest_tbl1_vw;
@@ -247,12 +367,16 @@ DROP VIEW mul_tbl_view;
 DELETE FROM f_test_tbl1;
 DELETE FROM f_test_tbl2;
 DELETE FROM f_numbers;
+DELETE FROM f_enum_t1;
 DROP FOREIGN TABLE f_test_tbl1;
 DROP FOREIGN TABLE f_test_tbl2;
 DROP FOREIGN TABLE f_numbers;
 DROP FOREIGN TABLE f_mysql_test;
 DROP FOREIGN TABLE f_enum_t1;
+DROP FOREIGN TABLE f_test_tbl3;
 DROP TYPE size_t;
+DROP TYPE enum_t1_size_t;
+DROP TYPE enum_t2_size_t;
 DROP FUNCTION test_param_where();
 DROP FUNCTION test_param_where2(int, text);
 DROP USER MAPPING FOR public SERVER mysql_svr;
